unit GSHHSReadForm;

{
V1.0.0  2000 May 22  First version
V1.0.2  2000 May 22  Add large lakes check
V1.0.4  2000 May 27  Add File, Open for choice of file
                     Add choice of views
V1.1.0  2001 Feb 11  Version for Delphi 5

Copyright  David J Taylor, Edinburgh
Web site:  www.satsignal.net
E-mail:    davidtaylor@writeme.com
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, RegistryVariables;

type
  TViewChoice = (vwBoth, vwLand, vwLargeLakes);

type
  TFormMain = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileExit: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    ButtonExit: TButton;
    ScrollBox1: TScrollBox;
    Image1: TImage;
    TimerLoad: TTimer;
    RadioGroupView: TRadioGroup;
    OpenDialog1: TOpenDialog;
    FileOpen: TMenuItem;
    N1: TMenuItem;
    View1: TMenuItem;
    ViewBoth: TMenuItem;
    ViewLand: TMenuItem;
    ViewLargeLakes: TMenuItem;
    procedure FormCreate (Sender: TObject);
    procedure FormDestroy (Sender: TObject);
    procedure FormShow (Sender: TObject);
    procedure FileExitClick (Sender: TObject);
    procedure About1Click (Sender: TObject);
    procedure ButtonExitClick (Sender: TObject);
    procedure FormCloseQuery (Sender: TObject; var CanClose: Boolean);
    procedure TimerLoadTimer (Sender: TObject);
    procedure RadioGroupViewClick (Sender: TObject);
    procedure FileOpenClick(Sender: TObject);
    procedure ViewBothClick(Sender: TObject);
    procedure ViewLandClick(Sender: TObject);
    procedure ViewLargeLakesClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    bmp: TBitmap;
    closing: boolean;
    gshhs_filename: TRegistryString;
    view: TViewChoice;
    procedure ReadGSHHSdata;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

uses AboutDisplay, GSHHSdata;

{$R *.DFM}

procedure TFormMain.FormCreate (Sender: TObject);
const
  cartog_resolution = 10;
begin
  bmp := TBitmap.Create;
  bmp.PixelFormat := pf24bit;
  bmp.Height := 180 * cartog_resolution;
  bmp.Width := 360 * cartog_resolution;
  Image1.Width := bmp.Width;
  Image1.Height := bmp.Height;
  gshhs_filename :=
    TRegistryString.CreateValue ('DataFile', 'C:\GSHHS\gshhs_l.b');
  view := vwBoth;
end;


procedure TFormMain.FormDestroy (Sender: TObject);
begin
  Image1.Picture.Bitmap := nil;
  bmp.Free;
  gshhs_filename.Free;
end;


procedure TFormMain.FormShow (Sender: TObject);
begin
  TimerLoad.Enabled := True;
end;


procedure TFormMain.FileExitClick (Sender: TObject);
begin
  Close;
end;


procedure TFormMain.About1Click (Sender: TObject);
begin
  AboutBoxDJT1.ShowModal;
end;


procedure TFormMain.ButtonExitClick (Sender: TObject);
begin
  Close;
end;


procedure TFormMain.ReadGSHHSdata;

  procedure skip_one_line (const hdr: TGSHHS;  var f: file);
  begin
    Seek (f, FilePos (f) + hdr.n * SizeOf (TGPoint))
  end;

  procedure read_one_line (const hdr: TGSHHS;  var f: file);
  const
    london_centric = True;
  var
    points: integer;
    point: integer;
    old_x, old_y: integer;
    xy: TGPOINT;
    bytes_read: integer;
    ax, ay: single;
    x, y: integer;
  begin
    points := hdr.n;
    old_x := -1;
    old_y := -1;
    for point := 1 to points do
      begin
      BlockRead (f, xy, SizeOf (xy), bytes_read);
      if bytes_read = SizeOf (xy) then
        begin
        PointSwap (xy);
        ax := xy.x * 1E-6;
        if london_centric then ax := ax + 180;
        if ax >= 360 then ax := ax - 360;
        x := Round (ax * bmp.Width / 360);
        ay := xy.y * 1E-6;
        y := Round ((90 - ay) * bmp.Height / 180);
        if point = 1
        then                // First point, simply move there
          begin
          bmp.Canvas.MoveTo (x, y);
          old_x := x;
          old_y := y;
          end
        else                  // There is a genuine move, or the last point
          if (x <> old_x) or (y <> old_y) or (point = points) then
            begin
            if Abs (x - old_x) < (bmp.Width div 4)
            then              // Move is within normal limits
              begin
              bmp.Canvas.LineTo (x, y);  // Plot the line
              old_x := x;     // Note the new pen co-ordinates
              old_y := y;
              end
            else              // Move crosses edge of map
              begin
              if x > old_x
              then            // Line is to the left
                begin         // Plot as two segments
                bmp.Canvas.LineTo (0, old_y);
                bmp.Canvas.MoveTo (bmp.Width, y);
                bmp.Canvas.LineTo (x, y);
                old_x := x;   // Record new pen co-ordinates
                old_y := y;
                end
              else
                begin         // Line is to the right
                bmp.Canvas.LineTo (bmp.Width, old_y);
                bmp.Canvas.MoveTo (0, y);
                bmp.Canvas.LineTo (x, y);
                old_x := x;   // Record new pen co-ordinates
                old_y := y;
                end
              end;
            end;
        end;
      end;
    end;

var
  f: file;
  hdr: TGSHHS;
  eof: boolean;
  bytes_read: integer;
begin
  bmp.Canvas.Rectangle (0, 0, bmp.Width, bmp.Height);
  try
    AssignFile (f, gshhs_filename.Value);
    Reset (f, 1);
    repeat
      BlockRead (f, hdr, SizeOf (hdr), bytes_read);
      eof := bytes_read <> SizeOf (hdr);
      if not eof then
        begin
        HeaderSwap (hdr);
        case view of
          vwBoth:
            if (hdr.level = lvLand) or
               ((hdr.level = lvLake) and (hdr.area > 10000))
              then read_one_line (hdr, f)
              else skip_one_line (hdr, f);
          vwLand:
            if hdr.level = lvLand
              then read_one_line (hdr, f)
              else skip_one_line (hdr, f);
          vwLargeLakes:
            if (hdr.level = lvLake) and (hdr.area > 10000)
              then read_one_line (hdr, f)
              else skip_one_line (hdr, f);
        else
          skip_one_line (hdr, f);
        end;
        end;
    until eof;
    CloseFile (f);
  except
  end;
  bmp.Modified := True;
  Caption := 'David Taylor''s GSHHS reader - ' + gshhs_filename.Value; 
end;


procedure TFormMain.FormCloseQuery (Sender: TObject; var CanClose: Boolean);
begin
  closing := True;
end;


procedure TFormMain.TimerLoadTimer (Sender: TObject);
begin
  TimerLoad.Enabled := False;
  if not closing then
    begin
    if not FileExists (gshhs_filename.Value) then
      if OpenDialog1.Execute then gshhs_filename.Value := OpenDialog1.Filename;
    ReadGSHHSdata;
    Image1.Picture.Bitmap := bmp;
    RadioGroupView.ItemIndex := 0;
    end;
end;


procedure TFormMain.RadioGroupViewClick (Sender: TObject);
begin
  case RadioGroupView.ItemIndex of
    0: begin
       Image1.Picture := nil;
       Image1.Width := ScrollBox1.Width;
       Image1.Height := ScrollBox1.Height;
       Image1.Stretch := True;
       ScrollBox1.HorzScrollBar.Visible := False;
       ScrollBox1.VertScrollBar.Visible := False;
       Image1.Picture.Bitmap := bmp;
       end;
    1: begin
       Image1.Picture := nil;
       Image1.Width := bmp.Width;
       Image1.Height := bmp.Height;
       Image1.Stretch := False;
       ScrollBox1.HorzScrollBar.Visible := True;
       ScrollBox1.VertScrollBar.Visible := True;
       Image1.Picture.Bitmap := bmp;
       end;
  end;
end;


procedure TFormMain.FileOpenClick (Sender: TObject);
begin
  if OpenDialog1.Execute then
    begin
    gshhs_filename.Value := OpenDialog1.Filename;
    ReadGSHHSdata;
    Image1.Picture.Bitmap := bmp;
    end;
end;

procedure TFormMain.ViewBothClick (Sender: TObject);
begin
  view := vwBoth;
  ViewBoth.Checked := True;
  ReadGSHHSdata;
  Image1.Picture.Bitmap := bmp;
end;

procedure TFormMain.ViewLandClick (Sender: TObject);
begin
  view := vwLand;
  ViewLand.Checked := True;
  ReadGSHHSdata;
  Image1.Picture.Bitmap := bmp;
end;

procedure TFormMain.ViewLargeLakesClick (Sender: TObject);
begin
  view := vwLargeLakes;
  ViewLargeLakes.Checked := True;
  ReadGSHHSdata;
  Image1.Picture.Bitmap := bmp;
end;

procedure TFormMain.FormResize (Sender: TObject);
begin
  if not closing then RadioGroupViewClick (Self);
end;

end.

